#http://hydrosheds.cr.usgs.gov/datadownload.php?reqdata=30bass

import urllib2
import xml.dom.minidom
from xml.dom.minidom import parse
import arcpy
import time
import sys
arcpy.env.overwriteOutput = True
arcpy.env.tileSize = "4096 4096"

# Check out any necessary licenses
try:
    if arcpy.CheckExtension("spatial") == "Available":
        arcpy.CheckOutExtension("spatial")
except:
    print "Spatial Analyst license is not available"
    sys.exit(0)
    
# Set Geoprocessing environments
arcpy.env.cartographicCoordinateSystem = ""

mxd=arcpy.mapping.MapDocument("CURRENT")
ProjectFolder=mxd.filePath.rsplit("\\",1)[0]
df= arcpy.mapping.ListDataFrames(mxd)[0]
dfExtentLL=df.extent.projectAs("WGS 1984")

BCubeFolder=ProjectFolder + "\\BCube"
basinlyrll=BCubeFolder+"\\BasinLL.shp"
publish=BCubeFolder+"\\publish.txt"
arcpy.AddMessage("Dan " + basinlyrll)

if not arcpy.Exists("Basin"):
    arcpy.AddMessage("You need to start with building a project, and running the watershed deliniation.")
    sys.exit(0)

Output_Spatial_Reference = dfExtentLL.spatialReference

Input_Spatial_Reference = arcpy.Describe("Basin").SpatialReference

arcpy.Project_management("Basin", basinlyrll, Output_Spatial_Reference)

desc=arcpy.Describe(basinlyrll)

lonmin=desc.extent.XMin
latmin=desc.extent.YMin
lonmax=desc.extent.XMax
latmax=desc.extent.YMax

clat=(latmax-latmin)/2+latmin
clon=(lonmax-lonmin)/2+lonmin

#clon= float(sys.argv[2])
#extent = float(sys.argv[3])
luselectlist =sys.argv[1]
luselect=luselectlist.split(';', 1)[0]
urlbase="http://gradlab4.bse.vt.edu:8080/gi-cat-10.0.2/"
if luselect == "Global_USGS_500m":
  UUID="UUID-8edb9f05-a2b7-470e-b133-365ff8449dc7"
  ProdName="LCType" 
  urlbase="http://gradlab4.bse.vt.edu:8080/gi-cat-10.0.2/"
elif luselect == "US_NLCD_30m":
  UUID="UUID-863a1955-5fb5-4911-aeb9-d48e38b12f26"
  ProdName="nlcd" 
  urlbase="http://gradlab4.bse.vt.edu:8080/gi-cat-10.0.2/"
else:
  arcpy.AddMessage("Product " + luselect + " is not available yet, sorry")
  sys.exit(0)
  
arcpy.AddMessage("Dan " + ProjectFolder + " " + str(lonmin) +" " +  str(lonmax) + " " + str(latmin) + " " + luselect)

arcpy.AddMessage("Dan " + BCubeFolder)
arcpy.env.workspace=BCubeFolder

bbox=str(lonmin)+","+str(latmin)+","+str(lonmax)+","+str(latmax) 
bboxspace=str(lonmin)+" "+str(latmin)+" "+str(lonmax)+" "+str(latmax) 
url=urlbase+"/services/opensearchsemanticenhanced?&reqID=npzmhOwDsB4bbF3UZ9EUPQ1R9b3ViTIV&radio_group=on&bbox=" + bbox + "&relation_radio_group=on&rel=overlaps&ct=1000&outputFormat=application%2Fatom%2Bxml&evtOrd=Time&sources="+UUID
arcpy.AddMessage(url)
dom=xml.dom.minidom.parse(urllib2.urlopen(url))
Topic=dom.getElementsByTagName('entry')
for node in Topic:
  onlinelist=node.getElementsByTagName('gmd:onLine')
  for olnode in onlinelist:
    linklist=olnode.getElementsByTagName('gmd:URL')
    protlist=olnode.getElementsByTagName('gco:CharacterString')
    tlist=node.getElementsByTagName('title')
    i=0
    for a in linklist:
      protocol=protlist[i].firstChild.data
      if 'WebCoverageService' in protocol:
       if 'protocol=OPeNDAP' in a.firstChild.data:
        url= a.firstChild.data + "SERVICE=WCS&VERSION=1.0.0";
        arcpy.AddMessage(url)
        wcslayer=tlist[0].firstChild.data
        print wcslayer
        arcpy.AddMessage(wcslayer)
        result = None
        while result is None:
          try: 
            if ('SRTMv3_3_S5' not in str(wcslayer)):
              wcslayer=str(wcslayer).lower().translate(None,"_")
              arcpy.AddMessage("getting "+wcslayer)
              arcpy.MakeWCSLayer_management(url,wcslayer,bboxspace) 
              arcpy.AddMessage("tiffing "+wcslayer)
              arcpy.CopyRaster_management(wcslayer,BCubeFolder + "\\" + wcslayer.split(".")[0] + ".tif")
              for lyr in arcpy.mapping.ListLayers(mxd, "", df):
                if '.nc' in str(lyr):
                  print "Success on:" + str(lyr)
                  arcpy.mapping.RemoveLayer(df,lyr)
              with open(publish, "a") as myfile:
                myfile.write("Landuse\n")
                myfile.write(url + "\n")
                myfile.close()
            result = True
          except:
            for lyr in arcpy.mapping.ListLayers(mxd, "", df):
              if '.nc' in str(lyr):
                print "failed on:" + str(lyr)
                arcpy.mapping.RemoveLayer(df,lyr)
            pass
      i=i+1


arcpy.RefreshActiveView()
arcpy.RefreshCatalog(arcpy.env.workspace)
arcpy.RefreshTOC()
lprodname=ProdName.lower().translate(None,"_")

arcpy.RefreshCatalog(arcpy.env.workspace)
rlist=arcpy.ListRasters(lprodname + '*',"TIF")
# Build the odd formatted input string.
inRasters=""
for grid in rlist:
   thisGrid = arcpy.env.workspace+'/'+grid
   #print thisGrid
   if len(inRasters) == 0:
     inRasters = thisGrid
   else:
     inRasters = inRasters + ';' + thisGrid
inRasters='"'+inRasters+'"'
arcpy.AddMessage(lprodname + " " + ",".join(rlist) + " " + arcpy.env.workspace)
arcpy.AddMessage(inRasters)

arcpy.MosaicToNewRaster_management(inRasters,arcpy.env.workspace, lprodname + ".tif","#","32_BIT_FLOAT", "#", 1, "MAXIMUM", "FIRST")

bboxcl=str(lonmin)+" "+str(latmin)+" "+str(lonmax)+" "+str(latmax) 

arcpy.AddMessage(arcpy.env.workspace+"/"+lprodname+".tif" +" "+ lprodname + "cl.tif")
arcpy.Clip_management(arcpy.env.workspace+"/"+lprodname+".tif",bboxcl,arcpy.env.workspace+"/"+lprodname + "cl.tif", "#", "#", "NONE")

utmzone=str(math.trunc((180+clon)//6+1))
utmCM=str((float(utmzone)-0.5)*6-180.0)

utmzone=Input_Spatial_Reference.exporttostring().split("UTM_Zone_",1)[1].split("',",1)[0]

if clat > 0:
  hemi="Northern"
  northing="0.0"
else:
  hemi="Southern"
  northing="10000000.0"

coordsys="Coordinate Systems/Projected Coordinate Systems/UTM/WGS 1984/"+hemi+" Hemisphere/WGS 1984 UTM Zone "+utmzone+".prj"

coordsys=Input_Spatial_Reference.exporttostring()
arcpy.AddMessage("CoordSys:"+coordsys)
Coordinate_System=Input_Spatial_Reference.exporttostring()
#Coordinate_System = "PROJCS['WGS_1984_UTM_Zone_"+utmzone+"',GEOGCS['GCS_WGS_1984',DATUM['D_WGS_1984',SPHEROID['WGS_1984',6378137.0,298.257223563]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Transverse_Mercator'],PARAMETER['False_Easting',500000.0],PARAMETER['False_Northing',"+northing+"],PARAMETER['Central_Meridian',"+utmCM+"],PARAMETER['Scale_Factor',0.9996],PARAMETER['Latitude_Of_Origin',0.0],UNIT['Meter',1.0]]"

arcpy.AddMessage("CoordSysNew:"+Coordinate_System)

arcpy.ProjectRaster_management(arcpy.env.workspace+"/"+lprodname+"cl.tif",arcpy.env.workspace+"/"+lprodname+utmzone+".tif", Coordinate_System,"NEAREST", "", "", "", "")

arcpy.CopyRaster_management(arcpy.env.workspace+"/"+lprodname+utmzone+".tif",arcpy.env.workspace+"/"+lprodname+utmzone+"8b.tif","#","#","#","NONE","NONE","8_BIT_UNSIGNED")
arcpy.BuildRasterAttributeTable_management(arcpy.env.workspace+"/"+lprodname+utmzone+"8b.tif","NONE")

rasters = arcpy.ListRasters(arcpy.env.workspace+"/"+lprodname+utmzone+"8b*","TIF")
raster=arcpy.env.workspace+"/"+lprodname+utmzone+"8b.tif"
arcpy.AddMessage("Adding "+raster)
arcpy.AddMessage(raster)
arcpy.AddMessage(arcpy.env.workspace)
arcpy.AddMessage(BCubeFolder)
addLayer = arcpy.mapping.Layer(raster)  
arcpy.mapping.AddLayer(df,addLayer)  
sr=arcpy.Describe(addLayer).spatialReference
arcpy.AddMessage("1dan")
df.spatialReference= sr
arcpy.RefreshActiveView()

