import urllib, htmllib, formatter, gzip, os, sys
import arcpy
from arcpy.sa import *
from arcpy import env
 
class LinksExtractor(htmllib.HTMLParser):
   def __init__(self, formatter):
      htmllib.HTMLParser.__init__(self, formatter)
      self.links = []
   def start_a(self, attrs):
      if len(attrs) > 0 :
         for attr in attrs :
            if attr[0] == "href":
                self.links.append(attr[1])
   def get_links(self):
      return self.links
format = formatter.NullFormatter()
htmlparser = LinksExtractor(format)

mapdoc=arcpy.mapping.MapDocument("CURRENT")

ProjectFolder=mapdoc.filePath.rsplit("\\",1)[0]
env.workspace = ProjectFolder
Projname=ProjectFolder.split("\\").pop()
BCubeFolder= ProjectFolder + "\\BCube"
if not(arcpy.Exists(BCubeFolder)):
   arcpy.AddMessage("Building the BCube Folder")
   arcpy.CreateFolder_management(ProjectFolder, "BCube")

ProjectDB = ProjectFolder + "\\" + Projname + ".mdb"
SWAT2012DB = ProjectFolder + "\\SWAT2012.mdb"
WatershedTable=ProjectDB + "\\ArcHydro\\Watershed"

projwgntblname="WGEN_"+Projname
projwgntbl=SWAT2012DB+"\\"+projwgntblname
userwgntblname="WGEN_US_FirstOrder"
userwgntbl=SWAT2012DB+"\\"+userwgntblname
if arcpy.Exists(projwgntbl):
   arcpy.Delete_management(projwgntbl)
   arcpy.AddMessage("Deleting " + projwgntbl)
arcpy.CreateTable_management(SWAT2012DB,projwgntblname,userwgntbl)


#f = open('workfile', 'w')
#f.write('0123456789abcdef')
#f.close
#ID,NAME,LAT,LONG,ELEVATION
#1,p423763,42.307,-76.250,346.000
#2,p423759,42.307,-75.938,311.000

varfiles=["pcp","rh","solar","tmp","wind"]
for varfile in varfiles:
   myfilename=BCubeFolder + "\\" + varfile+".txt"
   if arcpy.Exists(myfilename):
      arcpy.Delete_management(myfilename)
   with open(myfilename, "a") as myfile:
      myfile.write("ID,NAME,LAT,LONG,ELEVATION\n") 

arcpy.AddMessage("Dan2 " + ProjectDB)

cursor = arcpy.SearchCursor(WatershedTable)
for row in cursor:
   declat=row.getValue("Lat")
   declon=row.getValue("Long_")
   subbasin=row.getValue("Subbasin")
   subbasinstr=str(subbasin).zfill(5)
   elevation=row.getValue("Elev")
   for varfile in varfiles:
      myfilename=BCubeFolder + "\\" + varfile+".txt"
      stnfilestr=str(subbasin)+","+varfile+subbasinstr+","+str(declat)+","+str(declon)+","+str(elevation)+"\n"
      with open(myfilename, "a") as myfile:
         myfile.write(stnfilestr)

   for varfile in varfiles:
      myfilename=BCubeFolder + "\\" + varfile+subbasinstr+".txt"
      if arcpy.Exists(myfilename):
         arcpy.Delete_management(myfilename)

   arcpy.AddMessage("LatLon:"+str(declat)+","+str(declon))
   timeoff=0
   interppow=2
   url="http://www.cfsr.tamu-cornell.drfuka.org/swat-cfsr-v03.pl?lat="+str(declat)+"&lon="+str(declon)+"&timeoff="+str(timeoff)+"&interppow="+str(interppow)+"&.submit=Submit"
   arcpy.AddMessage("URL " + url)
   data = urllib.urlopen(url)
   htmlparser.feed(data.read())
   htmlparser.close()

   links = htmlparser.get_links()
   arcpy.AddMessage("URL " + links[0])
   urllib.urlretrieve(links[0], BCubeFolder+"/"+subbasinstr+".csv.gz")
# inefficient, but does not take much time.
   modays=len(gzip.open(BCubeFolder+"/"+subbasinstr+'.csv.gz', 'rb').readlines())/365.0
   f=gzip.open(BCubeFolder+"/"+subbasinstr+'.csv.gz', 'rb') 
   header = f.readline().rstrip().split(",")
   wxtbltmp=ProjectFolder+"\\TopoSWAT\\swat_topography.gdb\\wxtemplate"
   wxtblname="wx"+subbasinstr
   wxtbl=ProjectDB+"\\"+wxtblname
   if arcpy.Exists(wxtbl):
       arcpy.Delete_management(wxtbl)
       arcpy.AddMessage("Deleting " + wxtbl)
   arcpy.CreateTable_management(ProjectDB,wxtblname,wxtbltmp)
   startdate="blank"
   rows = arcpy.InsertCursor(wxtbl)
   prevwet=float(0)
   for line in f:
     row = rows.newRow()
     vals=line.rstrip().split(",")
     mo=int(vals[0].split("-")[1])
     if startdate=="blank":
        startdate="".join(vals[0].split("-"))
        for varfile in varfiles:
           myfilename=BCubeFolder + "\\" + varfile+subbasinstr+".txt"
           with open(myfilename, "a") as myfile:
              myfile.write(startdate+"\n")

     row.setValue("mo", mo)
     row.setValue("mdate", vals[0])
     row.setValue("tmx", vals[1])
     row.setValue("tmn", vals[2])
     myfilename=BCubeFolder + "\\"+"tmp"+subbasinstr+".txt"
     with open(myfilename, "a") as myfile:
       myfile.write(str(vals[1])+","+str(vals[2])+"\n")
     row.setValue("precip", vals[3])
     myfilename=BCubeFolder + "\\"+"pcp"+subbasinstr+".txt"
     with open(myfilename, "a") as myfile:
       myfile.write(str(vals[3])+"\n")
     row.setValue("wind", vals[4])
     myfilename=BCubeFolder + "\\"+"wind"+subbasinstr+".txt"
     with open(myfilename, "a") as myfile:
       myfile.write(str(vals[4])+"\n")
     row.setValue("avgrh", vals[5])
     myfilename=BCubeFolder + "\\"+"rh"+subbasinstr+".txt"
     with open(myfilename, "a") as myfile:
       myfile.write(str(vals[5])+"\n")
     row.setValue("maxrh", vals[6])
     row.setValue("minrh", vals[7])
     row.setValue("solar", vals[8])
     myfilename=BCubeFolder + "\\"+"solar"+subbasinstr+".txt"
     with open(myfilename, "a") as myfile:
       myfile.write(str(vals[8])+"\n")
     if float(vals[3]) < .35:
       row.setValue("precip", 0.0)
       row.setValue("dry", 1.0/modays)
       if prevwet > .35:
         row.setValue("wd", 1.0/modays)
     else:
       row.setValue("wet", 1.0/modays)
       if prevwet > .35:
         row.setValue("ww", 1.0/modays)
     rows.insertRow(row)
     prevwet=float(vals[3])
     del(row)
   del(rows)
   f.close()

 


# Calculate stats for wgn files
#if False: '''

   wxtblname="wx"+subbasinstr
   wgntblname="wgn"+subbasinstr
   wgntbl=ProjectDB+"/"+wgntblname
   if arcpy.Exists(wgntbl):
       arcpy.Delete_management(wgntbl)
       arcpy.AddMessage("Deleting " + wgntbl)
   medtblname="med"+subbasinstr
   medtbl=ProjectDB+"\\"+medtblname
   arcpy.Statistics_analysis(ProjectDB+"/"+wxtblname,wgntbl,"tmx MEAN;tmn MEAN;tmx STD;tmn STD;precip MEAN;precip STD;ww SUM;wd SUM;wet SUM;dry SUM;solar MEAN;wind MEAN;precip MAX;precip SUM;avgrh MEAN","mo")

   # Write to file, etc.
   # Calculate median and Skew
   #
   arcpy.env.overwriteOutput = True
   arcpy.env.qualifiedFieldNames = False
   # Median code borrowed and modified from: https://geonet.esri.com/thread/102159
   #Written By Caleb Mackey
   #4/17/2013
   #Updated to Python 3.x (ArcGIS Pro 1.0) by Katja Krivoruchko
   #4/16/2015
     
   def GetMedian(in_list):
     sorted_list = sorted(in_list)
     median = int(round(len(sorted_list) / 2))
     if len(sorted_list)%2==0:
       med_val = float(sorted_list[median-1]
             + sorted_list[median]) / 2
     else:
       med_val = sorted_list[median]
     return med_val

   case_field="[mo]"
   value_field="precip"
   values = {}
   if arcpy.Exists(medtbl):
          arcpy.Delete_management(medtbl)
          arcpy.AddMessage("Deleting " + medtbl)
   arcpy.CreateTable_management(ProjectDB,medtblname)
   arcpy.AddField_management(medtbl, "mo", "SHORT", 9, "", "", "", "NULLABLE", "REQUIRED", "") 
   arcpy.AddField_management(medtbl, "MEDIAN_precip", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "") 
   # Get Median UseValue for each station name
   wgnrows = arcpy.InsertCursor(medtbl)
   for st in range(1,13):
     print st
     query = '{0} = {1}'.format(case_field, st)
     use_vals = []
     rows=arcpy.SearchCursor(wxtbl,"mo="+str(st),"","precip")
     for row in rows:
       if row.getValue(value_field) != None:
           use_vals.append(row.getValue(value_field))
     if len(use_vals) > 0:
       median = GetMedian(use_vals)
       print median 
       wgnrow = wgnrows.newRow()
       wgnrow.setValue("mo", st)
       wgnrow.setValue("MEDIAN_precip", median)
       wgnrows.insertRow(wgnrow)
     del row
     del rows

   del wgnrow
   del wgnrows
   arcpy.AddMessage("Made it") 

   arcpy.AddField_management(wgntbl, "nTMPMX","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nTMPMN","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nTMPSTDMX","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nTMPSTDMN","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPCPMM","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPCPSTD","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPCPSKW","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPR_W1_","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPR_W2_","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nPCPD","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nRAINHHMX","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nSOLARAV","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nDEWPT","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.AddField_management(wgntbl, "nWNDAV","DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
   arcpy.JoinField_management(wgntbl, "mo", medtbl, "mo", "")
   # SKEW = 3(mean - median) / Standard Deviation
   arcpy.CalculateField_management(wgntbl, "nTMPMX", "[MEAN_tmx]", "VB", "")
   arcpy.AddMessage("Made it1") 
   arcpy.CalculateField_management(wgntbl, "nTMPMN", "[MEAN_tmn]", "VB", "")
   arcpy.AddMessage("Made it2") 
   arcpy.CalculateField_management(wgntbl, "nTMPSTDMX", "[STD_tmx]", "VB", "")
   arcpy.AddMessage("Made it3") 
   arcpy.CalculateField_management(wgntbl, "nTMPSTDMN", "[STD_tmn]", "VB", "")
   arcpy.AddMessage("Made it4") 
   arcpy.CalculateField_management(wgntbl, "nPCPMM", "[MEAN_precip]*30", "VB", "")
   arcpy.AddMessage("Made it1") 
   arcpy.CalculateField_management(wgntbl, "nPCPSTD", "[STD_precip]", "VB", "")
   arcpy.AddMessage("Made it2") 
   arcpy.CalculateField_management(wgntbl, "nPCPSKW", "3.0*([MEAN_precip]-[MEDIAN_precip])/[STD_precip]", "VB", "")
   arcpy.AddMessage("Made it3") 
   arcpy.CalculateField_management(wgntbl, "nPR_W1_", "[SUM_wd]/([SUM_dry]+.01)", "VB", "")
   arcpy.AddMessage("Made it4") 
#   arcpy.CalculateField_management(wgntbl, "nPR_W2_", "[SUM_ww]/[SUM_dry]", "VB", "")
   arcpy.CalculateField_management(wgntbl, "nPR_W2_", "[SUM_ww]/([SUM_wet]+.01)", "VB", "")
   arcpy.AddMessage("Made it1") 
   arcpy.CalculateField_management(wgntbl, "nPCPD", "[SUM_wet]", "VB", "")
   arcpy.AddMessage("Made it2") 
   arcpy.CalculateField_management(wgntbl, "nRAINHHMX", "[MAX_precip]/3", "VB", "")
   arcpy.AddMessage("Made it3") 
   arcpy.CalculateField_management(wgntbl, "nSOLARAV", "[MEAN_solar]", "VB", "")
   arcpy.AddMessage("Made it4") 
   arcpy.CalculateField_management(wgntbl, "nDEWPT", "[MEAN_avgrh]/100", "VB", "")
   arcpy.AddMessage("Made it") 
   arcpy.CalculateField_management(wgntbl, "nWNDAV", "[MEAN_wind]", "VB", "")
   arcpy.AddMessage("Made it") 
   wgntbltr=wgntbl+"tr"
   fieldstotr="nTMPMX TMPMX;nTMPMN TMPMN;nTMPSTDMX TMPSTDMX;nTMPSTDMN TMPSTDMN;nPCPMM PCPMM;nPCPSTD PCPSTD;nPCPSKW PCPSKW;nPR_W1_ PR_W1_;nPR_W2_ PR_W2_;nPCPD PCPD;nRAINHHMX RAINHHMX;nSOLARAV SOLARAV;nDEWPT DEWPT;nWNDAV WNDAV"
   if arcpy.Exists(wgntbltr):
     arcpy.Delete_management(wgntbltr)
     arcpy.AddMessage("Deleting " + wgntbltr)
   arcpy.TransposeFields_management(wgntbl,fieldstotr,wgntbltr,"param","value","mo")

   wgnhead=["STATE","STATION","LSTATION","ID","WLATITUDE","WLONGITUDE","WELEV","RAIN_YRS"]
   wgnparms=["TMPMX","TMPMN","TMPSTDMX","TMPSTDMN","PCPMM","PCPSTD","PCPSKW","PR_W1_","PR_W2_","PCPD","RAINHHMX","SOLARAV","DEWPT","WNDAV"]

   wgnrows = arcpy.InsertCursor(projwgntbl)
   wgnrow = wgnrows.newRow()
   wgnrow.setValue("STATION",subbasinstr)
   wgnrow.setValue("LSTATION",subbasinstr)
   wgnrow.setValue("ID",subbasin)
   wgnrow.setValue("WLATITUDE",declat)
   wgnrow.setValue("WLONGITUDE",declon)
   wgnrow.setValue("WELEV",elevation)
   wgnrow.setValue("RAIN_YRS",35.0)
   rows=arcpy.SearchCursor(wgntbltr)
   for row in rows:
     month=row.getValue("mo")
     param=row.getValue("param")
     value=row.getValue("value")
#     arcpy.AddMessage("SetValues" + param+str(month) + str(value)) 
     wgnrow.setValue(param+str(month),value)
   wgnrows.insertRow(wgnrow)
   del row
   del rows
   del wgnrow
del wgnrows


# Replace a layer/table view name with a path to a dataset (which can be a layer file) or create the layer/table view within the script
# The following inputs are layers or table views: "wx00003"
#   '''
arcpy.AddMessage("Dan3 " + env.workspace)
