# ---------------------------------------------------------------------------
# TopoSoil.py
# Created on: 2012-04-25
# Updated 12-05-02 02:45
# Description: 
# ---------------------------------------------------------------------------
# Import arcpy module
# Todo: Add in check for project file, topography database, and error out if missing
# Add in request for elevation/temperature sensitivity
# Add in request for if they want to alter swat2009/5 mdb, check if it exists in normal place
# Add in check for Required files in required locations.
# Limit processing to only those tasks that are needed for topo selections

import arcpy, arcgisscripting
import datetime
import sys, os, math
from arcpy.sa import *

# Check out any necessary licenses
try:
    if arcpy.CheckExtension("spatial") == "Available":
        arcpy.CheckOutExtension("spatial")
except:
    print "Spatial Analyst license is not available"
    sys.exit(0)
    
gp=arcgisscripting.create()
# Set Geoprocessing environments
arcpy.env.cartographicCoordinateSystem = ""

mapdoc=arcpy.mapping.MapDocument("CURRENT")
ProjectFolder=mapdoc.filePath.rsplit("\\",1)[0]
#ProjectFolder = sys.argv[1] 
#MDB_Filename = sys.argv[2] 
MDB_Overwrite = sys.argv[1] 
DoTI = sys.argv[2]
DoElev = sys.argv[3]
ElevIncr = sys.argv[4]
DoAspect = sys.argv[5]

ProjectDB = ProjectFolder + "\\" + ProjectFolder.split("\\").pop() + ".mdb"
MasterProgress=ProjectDB + "\\MasterProgress"

fields = ['SwatGDB']
with arcpy.da.SearchCursor(MasterProgress, fields) as cursor:
    for row in cursor:
       MDB_Filename = row[0]

gp.AddMessage("Dan " + ProjectFolder + MDB_Filename)

# Local variables:
FlowAcc = ProjectFolder + "\\RasterStore.mdb\\FlowAcc"
FlowAccR = Raster(FlowAcc)
Slope = ProjectFolder + "\\RasterStore.mdb\\Slope"
SlopeR = Raster(Slope)
DSMW = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb\\DSMW"
subs1_shp = ProjectFolder + "\\Watershed\\Shapes\\subs1.shp"
subs1_prj = ProjectFolder + "\\Watershed\\Shapes\\subs1.prj"
FlowDir = ProjectFolder + "\\RasterStore.mdb\\FlowDir"
ClipDem = ProjectFolder + "\\RasterStore.mdb\\ClipDem"
ClipDemR = Raster(ClipDem)
usersoil_GlobalTemplate = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb\\usersoil_GlobalTemplate"
usersoil_GTTemp = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb\\usersoil_GTTemp"
swat_topography_gdb = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb"
LMDB_Filename = ProjectFolder + "\\SWAT2012.mdb"


gb = ProjectFolder + ""
# Need to replace, but keeping to remember %workspace% reference needed in writing scripts.
# v_workspace_ = "%workspace%"
v_workspace_ = ProjectFolder
gp.AddMessage("Dan2 " + ProjectFolder)

TI = ProjectFolder + "\\RasterStore.mdb\\TI"
DSMWLatLon = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb\\DSMWLatLon"
DSMWLocal = ProjectFolder + "\\TopoSWAT\\swat_topography.gdb\\DSMWLocal"


DSMWRas = ProjectFolder + "\\RasterStore.mdb\\DSMWRas"
Topography = ProjectFolder + "\\RasterStore.mdb\\Topography"
TI_Class = ProjectFolder + "\\RasterStore.mdb\\TI_Class"
FlowDir8 = ProjectFolder + "\\RasterStore.mdb\\FlowDir8"
DEMIncr = ProjectFolder + "\\RasterStore.mdb\\DEMIncr"
usersoil_Local = ProjectFolder + "\\TopoSWAT\swat_topography.gdb\\usersoil_Local"
TopoSoil = ProjectFolder + "\\RasterStore.mdb\\TopoSoil"
Lookup1_dbf = ProjectFolder + "\\Lookup1.dbf"


# Process: Reclassify

if arcpy.Exists(FlowDir8):
    arcpy.Delete_management(FlowDir8)
    gp.AddMessage("Deleting " + FlowDir8)
arcpy.gp.Reclassify_sa(FlowDir, "VALUE", "1 1;2 2;4 3;8 4;16 5;32 6;64 7;128 8", FlowDir8, "DATA")
gp.AddMessage("Created " + FlowDir8)

# Process: Raster Calculator (3)

if arcpy.Exists(DEMIncr):
    arcpy.Delete_management(DEMIncr)
    gp.AddMessage("Deleting " + DEMIncr)

DEMIncr_R = Int(Int((Divide(ClipDemR,float(ElevIncr))))*(float(ElevIncr)/100))
DEMIncr_R.save(DEMIncr)
gp.AddMessage("Dan3 Built 100m elevation classes")

if arcpy.Exists(TI):
    arcpy.Delete_management(TI)
    gp.AddMessage("Deleting " + TI)

TI_R=Ln((FlowAccR+1)/(SlopeR+.001))
TI_R.save(TI)

gp.AddMessage("Dan3 " + TI)

# Process: Clip
if arcpy.Exists(DSMWLatLon):
    arcpy.Delete_management(DSMWLatLon)
    gp.AddMessage("Deleting " + DSMWLatLon)

arcpy.Clip_analysis(DSMW, subs1_shp, DSMWLatLon, "")
gp.AddMessage("Dan3 Clipped DSMW")


# Process: Project
if arcpy.Exists(DSMWLocal):
    arcpy.Delete_management(DSMWLocal)
    gp.AddMessage("Deleting " + DSMWLocal)

Input_Spatial_Reference = arcpy.Describe(DSMWLatLon).SpatialReference
gp.AddMessage("2" + Input_Spatial_Reference.datumName)

Output_Spatial_Reference = arcpy.Describe(Slope).SpatialReference
gp.AddMessage("2" + Output_Spatial_Reference.datumName)

try:
    arcpy.Project_management(DSMWLatLon, DSMWLocal, Output_Spatial_Reference)

except:
    # Process: Create Spatial Transforms
    CPUTime=str(os.times()[1])
    geoTransfmName = "SWATTopo"+CPUTime
    customGeoTransfm_path=os.getenv("appdata")+ "\\ESRI\\Desktop10.0\\ArcToolbox\\CustomTransformations\\"+geoTransfmName+".gtf"
    gp.AddMessage("DEBUG " + customGeoTransfm_path)
    if arcpy.Exists(customGeoTransfm_path):
        os.remove(customGeoTransfm_path)
        gp.AddMessage("Deleting " + customGeoTransfm_path)
                      
    customGeoTransfm = "GEOGTRAN[METHOD['Geocentric_Translation'],PARAMETER['X_Axis_Translation',''],PARAMETER['Y_Axis_Translation',''],PARAMETER['Z_Axis_Translation','']]"
    arcpy.CreateCustomGeoTransformation_management(geoTransfmName, Input_Spatial_Reference, Output_Spatial_Reference, customGeoTransfm)
    arcpy.Project_management(DSMWLatLon, DSMWLocal, Output_Spatial_Reference,geoTransfmName)
#except:
#    print "Please send us your project Watershed/Shapes/* files to debug this issue"
#    sys.exit(0)
    
gp.AddMessage("Dan3 Reprojected DSMW to project projection")

# Process: Polygon to Raster
if arcpy.Exists(DSMWRas):
    arcpy.Delete_management(DSMWRas)
    gp.AddMessage("Deleting " + DSMWRas)

arcpy.PolygonToRaster_conversion(DSMWLocal, "SNUM", DSMWRas, "CELL_CENTER", "NONE", Slope)
gp.AddMessage("Dan3 Rasterized DSMW")

# Process: Slice
if arcpy.Exists(TI_Class):
    arcpy.Delete_management(TI_Class)
    gp.AddMessage("Deleting " + TI_Class)

arcpy.gp.Slice_sa(TI, TI_Class, "10", "EQUAL_AREA", "1")
gp.AddMessage("Dan3 Sliced TIs into classes")

# Process: Combine
if arcpy.Exists(Topography):
    arcpy.Delete_management(Topography)
    gp.AddMessage("Deleting " + Topography)



# Select which fields you want/need
CombineFiles=DSMWRas
if (DoTI=="true"):
    CombineFiles = CombineFiles + ";" + TI_Class
if (DoElev=="true"):
    CombineFiles = CombineFiles + ";" + DEMIncr
if (DoAspect=="true"):
    CombineFiles = CombineFiles + ";" + FlowDir8


gp.AddMessage("Combine test "+DoTI+DoElev+DoAspect)
gp.AddMessage("Combine "+CombineFiles)
arcpy.gp.Combine_sa(CombineFiles, Topography)
gp.AddMessage("Combine "+CombineFiles)

# Process: Join Field
arcpy.JoinField_management(Topography, "DSMWRas", DSMWRas, "Value", "")

# Add default fields for those that are not selected.
if not(DoTI=="true"):
    arcpy.AddField_management(Topography, "TI_Class", "LONG", 9, "", "", "", "NULLABLE", "REQUIRED", "")
    arcpy.CalculateField_management(Topography, "TI_Class", "0", "VB", "")
if not(DoElev=="true"):
    arcpy.AddField_management(Topography, "DEMIncr", "LONG", 9, "", "", "", "NULLABLE", "REQUIRED", "")
    arcpy.CalculateField_management(Topography, "DEMIncr", "0", "VB", "")
if not(DoAspect=="true"):
    arcpy.AddField_management(Topography, "FlowDir8", "LONG", 9, "", "", "", "NULLABLE", "REQUIRED", "")
    arcpy.CalculateField_management(Topography, "FlowDir8", "0", "VB", "")


# Process: Join Field (2)
if arcpy.Exists(usersoil_GTTemp):
    arcpy.Delete_management(usersoil_GTTemp)
    gp.AddMessage("Deleting " + usersoil_GTTemp)
arcpy.Copy_management(usersoil_GlobalTemplate,usersoil_GTTemp)
arcpy.AddField_management(usersoil_GTTemp, "SEQNInt", "LONG", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(usersoil_GTTemp, "SEQNInt", "CInt([SEQN])", "VB", "")
arcpy.JoinField_management(Topography, "DSMWRas", usersoil_GTTemp, "SEQNInt", "")
arcpy.DeleteField_management(Topography, "SEQNInt")

# Process: Add Field
arcpy.AddField_management(Topography, "Name", "TEXT", "", "", "", "", "NULLABLE", "REQUIRED", "")
# Process: Calculate Field
arcpy.CalculateField_management(Topography, "NAME", "\"11\" + Right( String(2,\"0\") & [DEMIncr] ,2) + FormatNumber( [TI_Class] / 10 ,1) + \"TI\" + Right( String(2,\"0\") & [TI_Class] ,2) + \"A\" + FormatNumber([FlowDir8],0) + [SNAM]", "VB", "")

# Process: Calculate Field (2)
arcpy.CalculateField_management(Topography, "SNAM", "[NAME]", "VB", "")


# Process: Table Select
if arcpy.Exists(usersoil_Local):
    arcpy.Delete_management(usersoil_Local)
    gp.AddMessage("Deleting " + usersoil_Local)

arcpy.TableSelect_analysis(Topography, usersoil_Local, "")
DropFields = ["Value","Count","DSMWRAS","TI_Class","DEMIncr","FAOSOIL","Name","FlowDir8"]
arcpy.DeleteField_management(usersoil_Local, DropFields)
DropFields = ["Value_1","Count_1"]
arcpy.DeleteField_management(usersoil_Local, DropFields)

gp.AddMessage("Built " + usersoil_Local)

# Process: Copy Raster
if arcpy.Exists(TopoSoil):
    arcpy.Delete_management(TopoSoil)
    gp.AddMessage("Deleting " + TopoSoil)

arcpy.CopyRaster_management(Topography, TopoSoil, "", "", "", "NONE", "NONE", "32_BIT_SIGNED")
gp.AddMessage("Built " + TopoSoil)

# Process:Field
arcpy.DeleteField_management(TopoSoil, "Value_1,Count_1;FlowDir8;Count;DSMWRas;TI_Class;DEMIncr;FAOSOIL;NAME;MUID;SEQN;SNAM;S5ID;CMPPCT;NLAYERS;HYDGRP;SOL_ZMX;ANION_EXCL;SOL_CRK;TEXTURE;SOL_Z1;SOL_BD1;SOL_AWC1;SOL_K1;SOL_CBN1;CLAY1;SILT1;SAND1;ROCK1;SOL_ALB1;USLE_K1;SOL_EC1;SOL_Z2;SOL_BD2;SOL_AWC2;SOL_K2;SOL_CBN2;CLAY2;SILT2;SAND2;ROCK2;SOL_ALB2;USLE_K2;SOL_EC2;SOL_Z3;SOL_BD3;SOL_AWC3;SOL_K3;SOL_CBN3;CLAY3;SILT3;SAND3;ROCK3;SOL_ALB3;USLE_K3;SOL_EC3;SOL_Z4;SOL_BD4;SOL_AWC4;SOL_K4;SOL_CBN4;CLAY4;SILT4;SAND4;ROCK4;SOL_ALB4;USLE_K4;SOL_EC4;SOL_Z5;SOL_BD5;SOL_AWC5;SOL_K5;SOL_CBN5;CLAY5;SILT5;SAND5;ROCK5;SOL_ALB5;USLE_K5;SOL_EC5;SOL_Z6;SOL_BD6;SOL_AWC6;SOL_K6;SOL_CBN6;CLAY6;SILT6;SAND6;ROCK6;SOL_ALB6;USLE_K6;SOL_EC6;SOL_Z7;SOL_BD7;SOL_AWC7;SOL_K7;SOL_CBN7;CLAY7;SILT7;SAND7;ROCK7;SOL_ALB7;USLE_K7;SOL_EC7;SOL_Z8;SOL_BD8;SOL_AWC8;SOL_K8;SOL_CBN8;CLAY8;SILT8;SAND8;ROCK8;SOL_ALB8;USLE_K8;SOL_EC8;SOL_Z9;SOL_BD9;SOL_AWC9;SOL_K9;SOL_CBN9;CLAY9;SILT9;SAND9;ROCK9;SOL_ALB9;USLE_K9;SOL_EC9;SOL_Z10;SOL_BD10;SOL_AWC10;SOL_K10;SOL_CBN10;CLAY10;SILT10;SAND10;ROCK10;SOL_ALB10;USLE_K10;SOL_EC10")
arcpy.DeleteField_management(TopoSoil, "Value_1;Count_1;SOL_CAL1;SOL_CAL2;SOL_CAL3;SOL_CAL4;SOL_CAL5;SOL_CAL6;SOL_CAL7;SOL_CAL8;SOL_CAL9;SOL_CAL10;SOL_PH1;SOL_PH2;SOL_PH3;SOL_PH4;SOL_PH5;SOL_PH6;SOL_PH7;SOL_PH8;SOL_PH9;SOL_PH10")

# Process: Table to Table
if arcpy.Exists(Lookup1_dbf):
    arcpy.Delete_management(Lookup1_dbf)
    gp.AddMessage("Deleting " + Lookup1_dbf)

arcpy.TableToTable_conversion(Topography, ProjectFolder, "Lookup1.dbf", "", "Value \"Value\" false false false 9 Long 0 9 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\BNBSW0301\\RasterStore.mdb\\Topography\\Layer_1,Value,-1,-1,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,Value,-1,-1,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,Value,-1,-1;NAME \"NAME\" true false false 254 Text 0 0 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\BNBSW0301\\RasterStore.mdb\\Topography\\Layer_1,NAME,-1,-1,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,Name,-1,-1,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,Name,-1,-1;Count \"Count\" false false false 19 Double 0 0 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,Count,-1,-1;DSMWRas \"DSMWRas\" true false false 9 Long 0 9 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,DSMWRas,-1,-1;TI_Class \"TI_Class\" true false false 9 Long 0 9 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,TI_Class,-1,-1;DEMIncr \"DEMIncr\" true false false 9 Long 0 9 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,DEMIncr,-1,-1;FAOSOIL \"FAOSOIL\" true false false 14 Text 0 0 ,First,#,C:\\Documents and Settings\\dan1\\Desktop\\gb\\RasterStore.mdb\\Topography\\Layer_1,FAOSOIL,-1,-1", "")

# Process: Add Field (2)
arcpy.AddField_management(Lookup1_dbf, "vtemp", "TEXT", "", "", "5", "", "NON_NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management(Lookup1_dbf, "vtemp", "[Value]", "VB", "")
arcpy.DeleteField_management(Lookup1_dbf, "Value")
arcpy.AddField_management(Lookup1_dbf, "Value", "TEXT", "", "", "5", "", "NON_NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management(Lookup1_dbf, "Value", "[vtemp]", "VB", "")
arcpy.DeleteField_management(Lookup1_dbf, "vtemp")
arcpy.AddField_management(Lookup1_dbf, "ntemp", "TEXT", "", "", "50", "", "NON_NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management(Lookup1_dbf, "ntemp", "[Name]", "VB", "")
arcpy.DeleteField_management(Lookup1_dbf, "NAME")
arcpy.AddField_management(Lookup1_dbf, "Name", "TEXT", "", "", "50", "", "NON_NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management(Lookup1_dbf, "NAME", "[ntemp]", "VB", "")
arcpy.DeleteField_management(Lookup1_dbf, "ntemp")

arcpy.DeleteField_management(Lookup1_dbf, "FlowDir8;Count;DSMWRas;TI_Class;DEMIncr;FAOSOIL;MUID;SEQN;SNAM;S5ID;CMPPCT;NLAYERS;HYDGRP;SOL_ZMX;ANION_EXCL;SOL_CRK;TEXTURE;SOL_Z1;SOL_BD1;SOL_AWC1;SOL_K1;SOL_CBN1;CLAY1;SILT1;SAND1;ROCK1;SOL_ALB1;USLE_K1;SOL_EC1;SOL_Z2;SOL_BD2;SOL_AWC2;SOL_K2;SOL_CBN2;CLAY2;SILT2;SAND2;ROCK2;SOL_ALB2;USLE_K2;SOL_EC2;SOL_Z3;SOL_BD3;SOL_AWC3;SOL_K3;SOL_CBN3;CLAY3;SILT3;SAND3;ROCK3;SOL_ALB3;USLE_K3;SOL_EC3;SOL_Z4;SOL_BD4;SOL_AWC4;SOL_K4;SOL_CBN4;CLAY4;SILT4;SAND4;ROCK4;SOL_ALB4;USLE_K4;SOL_EC4;SOL_Z5;SOL_BD5;SOL_AWC5;SOL_K5;SOL_CBN5;CLAY5;SILT5;SAND5;ROCK5;SOL_ALB5;USLE_K5;SOL_EC5;SOL_Z6;SOL_BD6;SOL_AWC6;SOL_K6;SOL_CBN6;CLAY6;SILT6;SAND6;ROCK6;SOL_ALB6;USLE_K6;SOL_EC6;SOL_Z7;SOL_BD7;SOL_AWC7;SOL_K7;SOL_CBN7;CLAY7;SILT7;SAND7;ROCK7;SOL_ALB7;USLE_K7;SOL_EC7;SOL_Z8;SOL_BD8;SOL_AWC8;SOL_K8;SOL_CBN8;CLAY8;SILT8;SAND8;ROCK8;SOL_ALB8;USLE_K8;SOL_EC8;SOL_Z9;SOL_BD9;SOL_AWC9;SOL_K9;SOL_CBN9;CLAY9;SILT9;SAND9;ROCK9;SOL_ALB9;USLE_K9;SOL_EC9;SOL_Z10;SOL_BD10;SOL_AWC10;SOL_K10;SOL_CBN10;CLAY10;SILT10;SAND10;ROCK10;SOL_ALB10;USLE_K10;SOL_EC10")
arcpy.DeleteField_management(Lookup1_dbf, "Value_1;Count_1;SOL_CAL1;SOL_CAL2;SOL_CAL3;SOL_CAL4;SOL_CAL5;SOL_CAL6;SOL_CAL7;SOL_CAL8;SOL_CAL9;SOL_CAL10;SOL_PH1;SOL_PH2;SOL_PH3;SOL_PH4;SOL_PH5;SOL_PH6;SOL_PH7;SOL_PH8;SOL_PH9;SOL_PH10")

mxd=arcpy.mapping.MapDocument("CURRENT")
ProjectFolder=mxd.filePath.rsplit("\\",1)[0]
df= arcpy.mapping.ListDataFrames(mxd)[0]
TopoSoil = ProjectFolder + "\\RasterStore.mdb\\TopoSoil"
addLayer = arcpy.mapping.Layer(TopoSoil)
arcpy.mapping.AddLayer(df,addLayer)
arcpy.RefreshActiveView()

if arcpy.Exists(DSMWLatLon):
    arcpy.Delete_management(DSMWLatLon)
    gp.AddMessage("Deleting " + DSMWLatLon)

if arcpy.Exists(DSMWLocal):
    arcpy.Delete_management(DSMWLocal)
    gp.AddMessage("Deleting " + DSMWLocal)

if arcpy.Exists(usersoil_GTTemp):
    arcpy.Delete_management(usersoil_GTTemp)
    gp.AddMessage("Deleting " + usersoil_GTTemp)

if (MDB_Overwrite=="true"):
    now=datetime.datetime.now()
    usersoil_mbdfile=MDB_Filename+"\\usersoil"
    usersoil_mbdfilebak=usersoil_mbdfile+now.strftime("%Y%m%d%H%M%S")
    gp.AddMessage("Dan " + usersoil_mbdfile)
    gp.AddMessage("Dan " + usersoil_mbdfilebak)

    arcpy.Copy_management(usersoil_mbdfile,usersoil_mbdfilebak)
    arcpy.Delete_management(usersoil_mbdfile)
    arcpy.Copy_management(usersoil_Local,usersoil_mbdfile)

# Add usersoil table to local SWAT2012.mdb file 
    usersoil_lmbdfile=LMDB_Filename+"\\usersoil"
    usersoil_lmbdfilebak=usersoil_lmbdfile+now.strftime("%Y%m%d%H%M%S")
    gp.AddMessage("Dan" + usersoil_lmbdfile)

    arcpy.Copy_management(usersoil_lmbdfile,usersoil_lmbdfilebak)
    arcpy.Delete_management(usersoil_lmbdfile)
    arcpy.Copy_management(usersoil_Local,usersoil_lmbdfile)
    arcpy.Delete_management(usersoil_Local)
    

