# ---------------------------------------------------------------------------
# TopoSoil.py
# Created on: 2012-04-25
# Updated 12-05-02 02:45
# Description: 
# ---------------------------------------------------------------------------
# Import arcpy module
# Todo: Add in check for project file, topography database, and error out if missing
# Add in request for elevation/temperature sensitivity
# Add in request for if they want to alter swat2009/5 mdb, check if it exists in normal place
# Add in check for Required files in required locations.
# Limit processing to only those tasks that are needed for topo selections

import arcpy, arcgisscripting
import datetime
import sys, os, math
from arcpy.sa import *

# Check out any necessary licenses
try:
    if arcpy.CheckExtension("spatial") == "Available":
        arcpy.CheckOutExtension("spatial")
except:
    print "Spatial Analyst license is not available"
    sys.exit(0)
    
gp=arcgisscripting.create()
# Set Geoprocessing environments
arcpy.env.cartographicCoordinateSystem = ""

mapdoc=arcpy.mapping.MapDocument("CURRENT")
ProjectFolder=mapdoc.filePath.rsplit("\\",1)[0]
DoCN= sys.argv[1]
DoSoilZ= sys.argv[2]
SoilZFactor = sys.argv[3]
DoClay= sys.argv[4]
ClayFactor = sys.argv[5]
DoSand= sys.argv[6]
SandFactor = sys.argv[7]

gp.AddMessage("Dan " + ProjectFolder)

# Local variables:
ProjectDB = ProjectFolder + "\\" + ProjectFolder.split("\\").pop() + ".mdb"
gp.AddMessage("Dan2 " + ProjectDB)
solTable=ProjectDB + "\\sol"
hruTable=ProjectDB + "\\hru"
solStatsTable=ProjectDB + "\\drfstats"
mgt1Table=ProjectDB + "\\mgt1"

if not arcpy.Exists(solTable):
    sys.exit("Appears you have not written tables")

if arcpy.Exists(solStatsTable):
    arcpy.Delete_management(solStatsTable)

statsFields = [["NLAYERS", "MAX"]]
arcpy.Statistics_analysis(solTable, solStatsTable, statsFields)
cursor = arcpy.SearchCursor(solStatsTable)
for row in cursor:
    max_nlayers=row.getValue("MAX_NLAYERS")

gp.AddMessage("Max Soil Layers:"+str(max_nlayers))
#TopoSoil = ProjectFolder + "\\RasterStore.mdb\\TopoSoil"

# Distribute CNs
gp.AddMessage("Distributing CNs" )
arcpy.AddField_management(mgt1Table, "ASt", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(mgt1Table, "ASt", "1.1-CDbl(Mid([SOIL],5,3))", "VB", "")
arcpy.AddField_management(mgt1Table, "CN2temp", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(mgt1Table, "CN2temp", "2540/(25.4+((2540/[CN2]-25.4))*(2*(Sqr(1.0-([ASt]-.1))-Sqr(1.0-[ASt]))/.1-1.0))", "VB", "")
arcpy.CalculateField_management(mgt1Table, "CN2", "[CN2temp]", "VB", "")
arcpy.DeleteField_management(mgt1Table, "ASt;CN2temp")

# Distribute Soil Parameters
gp.AddMessage("Working on Soils Parameters" )
arcpy.AddField_management(solTable, "TIScale", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(solTable, "TIScale", "CDbl(Mid([SOIL],5,3))", "VB", "")
arcpy.AddField_management(solTable, "l1500kpa", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.AddField_management(solTable, "l33kpa", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.AddField_management(solTable, "lambda", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")

# Distributing Soil Depths
gp.AddMessage("Distributing Soil Depths" )
arcpy.AddField_management(solTable, "ScaleFactor", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(solTable, "ScaleFactor", SoilZFactor, "VB", "")
arcpy.AddField_management(solTable, "SolZParTemp", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(solTable, "SolZParTemp", "1.0+[ScaleFactor]*[TIScale]*2.0-.5*[ScaleFactor]*2.0", "VB", "")
arcpy.CalculateField_management(solTable, "SOL_ZMX", "[SOL_ZMX]*[SolZParTemp]", "VB", "")
arcpy.CalculateField_management(solTable, "ScaleFactor", ClayFactor, "VB", "")
arcpy.AddField_management(solTable, "SolCParTemp", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(solTable, "SolCParTemp", "1.0+[ScaleFactor]*[TIScale]*2.0-.5*[ScaleFactor]*2.0", "VB", "")
arcpy.AddField_management(solTable, "SolSParTemp", "DOUBLE", 9, "", "", "", "NULLABLE", "REQUIRED", "")
arcpy.CalculateField_management(solTable, "ScaleFactor", SandFactor, "VB", "")
arcpy.CalculateField_management(solTable, "SolSParTemp", "1.0+[ScaleFactor]*[TIScale]*2.0-.5*[ScaleFactor]*2.0", "VB", "")

# Set DEP_IMP = SOL_ZMX for VSA Hydrology
#DRF Amy found bug here, the HRU columns are not same informaiton, so sub+hru is needed in join.

gp.AddMessage("Setting DEP_IMP = SOL_ZMX for VSA.. should skip later if desired." )
arcpy.JoinField_management(hruTable,"HRU",solTable,"HRU","SOL_ZMX")
arcpy.CalculateField_management(hruTable, "DEP_IMP", "[SOL_ZMX]*1.2", "VB", "")
arcpy.DeleteField_management(hruTable, "SOL_ZMX")
gp.AddMessage("Finished Setting DEP_IMP = SOL_ZMX for VSA.. should skip later if desired." )

# Distribute Sand and Clay, calculate silt
#for tvar in ["1","2","3","4","5","6","7","8","9","10"]:
for itvar in range(1, int(max_nlayers)+1):
        tvar=str(itvar)
	gp.AddMessage("Distributing Soil Texture through Soil Layers:" + tvar )
	gp.AddMessage("Soil Layer:"+tvar)
	lsol_z="[SOL_Z"+tvar+"]*[SolZParTemp]"
	lclay="[CLAY"+tvar+"]*[SolCParTemp]"
	lsol_cbn="[SOL_CBN" + tvar + "]*[SolCParTemp]"
	lsand="[SAND"+tvar+"]*[SolSParTemp]"
	lsilt="100.0-[SAND"+tvar+"]-[CLAY"+tvar+"]"
	l1500kpa="-0.024*[SAND"+tvar+"]/100.0+0.487*[CLAY"+tvar+"]/100.0+0.006*[SOL_CBN"+tvar+"]/100.0+0.005*[SAND"+tvar+"]/100.0*[SOL_CBN"+tvar+"]/100.0-0.013*[CLAY"+tvar+"]/100.0*[SOL_CBN"+tvar+"]/100.0+0.068*[SAND"+tvar+"]/100.0*[CLAY"+tvar+"]/100.0+0.031"
	l33kpa ="-0.251*[SAND"+tvar+"]/100.0+0.195*[CLAY"+tvar+"]/100.0+0.011*[SOL_CBN"+tvar+"]/100.0+0.006*[SAND"+tvar+"]/100.0*[SOL_CBN"+tvar+"]/100.0-0.027*[CLAY"+tvar+"]/100.0*[SOL_CBN"+tvar+"]/100.0+0.452*[SAND"+tvar+"]/100.0*[CLAY"+tvar+"]/100.0+0.299"
	gp.AddMessage(lsol_z)
	arcpy.CalculateField_management(solTable, "SOL_Z"+tvar, lsol_z, "VB", "")
	gp.AddMessage(lclay)
	arcpy.CalculateField_management(solTable, "CLAY"+tvar, lclay, "VB", "")
	gp.AddMessage(lsol_cbn)
	arcpy.CalculateField_management(solTable, "SOL_CBN"+tvar, lsol_cbn, "VB", "")
	gp.AddMessage(lsand)
	arcpy.CalculateField_management(solTable, "SAND"+tvar, lsand, "VB", "")
	gp.AddMessage(lsilt)
	arcpy.CalculateField_management(solTable, "SILT"+tvar, lsilt, "VB", "")
	gp.AddMessage(l1500kpa)
	arcpy.CalculateField_management(solTable, "l1500kpa", l1500kpa, "VB", "")
	gp.AddMessage(l33kpa)
	arcpy.CalculateField_management(solTable, "l33kpa", l33kpa, "VB", "")
	gp.AddMessage("lambda")
	arcpy.CalculateField_management(solTable, "lambda", "(Log([l33kpa])-Log([l1500kpa]))/(Log(1500)-Log(33))", "VB", "")
	gp.AddMessage("SOL_K")
	arcpy.CalculateField_management(solTable, "SOL_K"+tvar, "1930.0*(.48-[l33kpa])^(3-[lambda])", "VB", "")
	gp.AddMessage("SOL_AWC")
	arcpy.CalculateField_management(solTable, "SOL_AWC"+tvar, "([l33kpa]-[l1500kpa])", "VB", "")

arcpy.DeleteField_management(solTable, "TIScale")
arcpy.DeleteField_management(solTable, "l1500kpa")
arcpy.DeleteField_management(solTable, "l33kpa")
arcpy.DeleteField_management(solTable, "lambda")
arcpy.DeleteField_management(solTable, "ScaleFactor")
arcpy.DeleteField_management(solTable, "SolZParTemp")
arcpy.DeleteField_management(solTable, "SolCParTemp")
arcpy.DeleteField_management(solTable, "SolSParTemp")

gp.AddMessage("Dan Maybe success?" )

# TODO's
# Calculate porosity via saxton to replace the .48 in the SOL_K calculation
# LAT_TTIME in hru ... given medosa/walter/brutsaert ksat lat = 10-100x soil ksat, we allow this as an option to be implemented
# DEP_IMP in hrus, set to soil_mxz
# Saxton AWC = 33kpa-1500kpa
# [SAND1] = SAND, [CLAY1]=CLAY, [SOL_CBN1]=SOL_CBN
# 1500kpa =-0.024*[SAND1]+0.487*[CLAY1]+0.006*[SOL_CBN1]+0.005*[SAND1]*[SOL_CBN1]-0.013*[CLAY1]*[SOL_CBN1]+0.068*[SAND1]*[CLAY1]+0.031
# 33kpa =-0.251*[SAND1]+0.195*[CLAY1]+0.011*[SOL_CBN1]+0.006*[SAND1]*[SOL_CBN1]-0.027*[CLAY1]*[SOL_CBN1]+0.452*[SAND1]*[CLAY1]+0.299
# Saxton KSat
# lamda= (LN(33kpa)-LN(1500kpa))/(LN(1500)-LN(33))
# ksat(mm/hr)=1930*(33kpa)^(3-lambda)*rockadjust
# Irrigation set at % of Basin PET from reach file AreaC in each reach
# Lateral flow time distribute Lag Time
# Ksat = 100xKsat, mendosa/walter 2003 and Brutsaert, W., Lopez, J.P., 1998. Basin-scale geohydrologic drought flow features of riparian aquifers in the southern great plains.  Water Resour. Res. 34 (2), 233 – 240
# Soil porosity Adjusted 33kpa porosity + 33kpa porous moisture=0.278*F18+0.034*G18+0.022*H18-0.018*F18*H18-0.027*G18*H18-0.584*F18*G18+0.078 
# Soil BD = (1-porosity)*2.65

